
#include <zephyr/drivers/gpio.h>

#include <zephyr/drivers/SPi.h>
#include "mfrc522.h"

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led1)
uint8_t ID[5];
/*
 * A build error on this line means your board is unsupported.
 * See the sample documentation for information on how to fix this.
 */
static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED0_NODE, gpios);

TM_MFRC522_STS_T TM_MFRC522_ReadBlock(uint8_t blockAddr, uint8_t *buffer, uint16_t *bufferLen)
{
        TM_MFRC522_STS_T status;
        uint8_t cmd[2];

        cmd[0] = PICC_READ; // Lệnh đọc block (0x30)
        cmd[1] = 0;         // Địa chỉ block muốn đọc

        // Gửi lệnh đọc
        status = TM_MFRC522_ToCard(PCD_TRANSCEIVE, cmd, 2, buffer, bufferLen);

        // Kiểm tra kết quả
        if (status != MI_OK)
        {
                printk("=========================Reading block %d failed!status %d \n", blockAddr, status);
                return MI_ERR;
        }

        printk("Read Block %d: ", blockAddr);
        for (int i = 0; i < 16; i++)
        {
                printk("%02X ", buffer[i]);
        }
        printk("\n");

        return MI_OK;
}

TM_MFRC522_STS_T TM_MFRC522_ReadUID(uint8_t *uid)
{
        uint8_t tagType[2];
        TM_MFRC522_STS_T status;

        // 🔹 Gửi yêu cầu kiểm tra thẻ
        status = MFRC522_Request(0x26, tagType); // 0x26 = REQA (0x52 nếu dùng WUPA)
        if (status != MI_OK)
        {
                return status;
        }

        // 🔹 Gửi lệnh Anti-Collision để lấy UID
        status = TM_MFRC522_Anticoll(uid);
        if (status != MI_OK)
        {
                return status;
        }

        // 🔹 Gửi lệnh Select thẻ
        // status = MFRC522_Authenticate(0x93, 0x00, NULL, uid); // Chỉ cần xác nhận UID
        // if (status != MI_OK)
        // {
        //         return status;
        // }

        return MI_OK;
}

#include "MFRC522.h"

uint8_t MFRC522_SelectTag(uint8_t *uid)
{
        uint8_t buffer[9]; // Buffer dữ liệu gửi đi
        uint16_t recvBits; // Số bit nhận được
        uint8_t status;
        uint8_t i;
        uint8_t sak; // Select Acknowledge (SAK)

        // Gửi lệnh SELECT với mã thẻ (0x93 cho MIFARE 1K)
        buffer[0] = PICC_SElECTTAG;
        buffer[1] = 0x70; // Byte chỉ định mức SELECT
        for (i = 0; i < 5; i++)
        {
                buffer[i + 2] = uid[i]; // Copy UID vào buffer
        }

        // Tính CRC-A (bắt buộc để xác thực)
        TM_MFRC522_CalculateCRC(buffer, 7, &buffer[7]);

        // Gửi lệnh SELECT qua giao tiếp với thẻ
        status = TM_MFRC522_ToCard(PCD_TRANSCEIVE, buffer, 9, buffer, &recvBits);

        // Kiểm tra phản hồi từ thẻ
        if ((status == MI_OK) && (recvBits == 24))
        {
                sak = buffer[0]; // Lưu Select Acknowledge (SAK)
                return sak;
        }

        return 0; // Lỗi hoặc thẻ không phản hồi đúng
}

int main(void)
{
        int ret;
        SPI_Init();
        TM_MFRC522_Init();
        if (!gpio_is_ready_dt(&led))
        {
                return 0;
        }

        ret = gpio_pin_configure_dt(&led, GPIO_OUTPUT_ACTIVE);
        if (ret < 0)
        {
                return 0;
        }
        uint8_t version = TM_MFRC522_RdReg(MFRC522_REG_VERSION);
        printk("MFRC522 Version: 0x%X\n", version);
        // uint8_t buffer[16];
        // uint8_t len;
        // uint8_t status = TM_MFRC522_ReadBlock(7, buffer, &len);

        // if (status == MI_OK)
        // {
        //         printk("Sector Trailer (Block 7): ");
        //         for (int i = 0; i < 16; i++)
        //         {
        //                 printk("%02X ", buffer[i]);
        //         }
        //         printk("\n");
        // }
        // while (1)
        // {
        //         uint8_t keyA[6] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}; // Key A mặc định
        //         uint8_t keyB[6] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}; // Key B mặc định

        //         // Xác thực với Key A
        //         printk("Authentication Start\n");
        //         TM_MFRC522_STS_T status = TM_MFRC522_Auth(PICC_AUTHENT1A, 4, keyA, ID);
        //         if (status != MI_OK)
        //         {
        //                 printk("Authentication failed with Key A!\n");
        //                 // return MI_ERR;
        //         }
        //         printk("Authentication successful with Key A!\n");

        //         // Sau khi xác thực thành công, bạn có thể tiếp tục đọc Block 7
        //         uint8_t buffer[16];
        //         uint16_t len;
        //         printk("Read data on block Start\n");
        //         status = TM_MFRC522_ReadBlock(0, buffer, &len);
        //         if (status == MI_OK)
        //         {
        //                 printk("Block 0 data: ");
        //                 for (int i = 0; i < 16; i++)
        //                 {
        //                         printk("%02X ", buffer[i]);
        //                 }
        //                 printk("\n");
        //         }
        //         printk("End Read\n");
        //         k_sleep(K_MSEC(1000));
        // }
        if (TM_MFRC522_ReadUID(ID) == MI_ERR)
                printk("UID-1: %02X-%02X-%02X-%02X-%02X\r\n", ID[0], ID[1], ID[2], ID[3], ID[4]);
        if (TM_MFRC522_ReadUID(ID) == MI_ERR)
                printk("UID-2: %02X-%02X-%02X-%02X-%02X\r\n", ID[0], ID[1], ID[2], ID[3], ID[4]);
        if (TM_MFRC522_ReadUID(ID) == MI_ERR)
                printk("UID-3: %02X-%02X-%02X-%02X-%02X\r\n", ID[0], ID[1], ID[2], ID[3], ID[4]);
        uint8_t key[4][6] = {
            {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}, // Key 0: FF FF FF FF FF FF (Mặc định)
            {0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // Key 1: 00 00 00 00 00 00
            {0xD3, 0xF7, 0xD3, 0xF7, 0xD3, 0xF7}, // Key 2: D3 F7 D3 F7 D3 F7
            {0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5}  // Key 3: A0 A1 A2 A3 A4 A5
        };

        // uint8_t uid[4] = {0xF3, 0x50, 0xBA, 0x02}; // UID của thẻ (cần lấy từ bước đọc UID)
        uint8_t block_addr = 4;                   // Block cần xác thực
        uint8_t type_tag[2] = {0};
        while (1)
        {
                k_sleep(K_MSEC(1000));
                TM_MFRC522_Halt();
                printk("=================================== MFRC522_Request\n");
                if (MFRC522_Request(0x26, type_tag) != MI_OK)
                {
                        printk("ERROR: MFRC522_Request failed! No card detected.\n");
                        continue;
                }
                printk("Card detected! Type: 0x%X 0x%X\n", type_tag[0], type_tag[1]);

                printk("=================================== TM_MFRC522_Anticoll\n");
                if (TM_MFRC522_Anticoll(ID) != MI_OK)
                {
                        printk("ERROR: TM_MFRC522_Anticoll failed! Unable to get UID.\n");
                        continue;
                }
                printk("Card UID: %02X %02X %02X %02X\n", ID[0], ID[1], ID[2], ID[3]);
                TM_MFRC522_Halt();

                k_sleep(K_MSEC(500));
                printk("=================================== MFRC522_SelectTag\n");
                uint8_t sak = MFRC522_SelectTag(ID);
                if (sak == 0)
                {
                        printk("ERROR: MFRC522_SelectTag failed! No response from card.\n");
                        continue;
                }
                printk("Card SAK: 0x%X\n", sak);
                TM_MFRC522_Halt();
                k_sleep(K_MSEC(500));
                int auth_status = MFRC522_Authenticate(PICC_AUTHENT1A, block_addr, key[0], ID);
                while (auth_status != MI_OK)
                {
                        if (TM_MFRC522_ReadUID(ID) == MI_ERR)
                                printk("UID-3: %02X-%02X-%02X-%02X-%02X\r\n", ID[0], ID[1], ID[2], ID[3], ID[4]);
                        printk("Retrying Authentication...\n");
                        TM_MFRC522_AntennaOn();
                        k_sleep(K_MSEC(1000));
                        printk("=================================== MFRC522_Authenticate\n");
                        auth_status = MFRC522_Authenticate(PICC_AUTHENT1A, block_addr, key[0], ID);
                        if(auth_status == MI_OK)
                        {
                                printk("========================== MFRC522_Authenticate Fail\n");
                        }
                        TM_MFRC522_AntennaOff();
                        k_sleep(K_MSEC(1000));
                        TM_MFRC522_AntennaOn();
                        uint8_t buffer[16];
                        uint16_t len;
                        printk("Read data onblock\n");
                        uint8_t status = TM_MFRC522_ReadBlock(block_addr, buffer, &len);
                }
                return 0;
        }
}